import {
  Metadata, Loader, IEventParam, Datasource, I18n, IDictionary, Config, Http, Mask,
} from '@zeedhi/core';

import {
  GridEditable, Card, DialogService, TextInput, Form, ModalService, Modal, Button, Select,
} from '@zeedhi/common';

import {
  PmeMask,
} from '@/config';

export class Ger01000UnidadeAdicionalProdutoController {
  public gridEditController = Loader.getInstance('GridEditableController');

  public crudController = Loader.getInstance('CrudController');
  
  public utilities = Loader.getInstance('Utilities');
  
  public modalController = Loader.getInstance('ModalController');

  private modalUnidade!: Modal;
  
  public modalConversao!: Modal;
  
  private conversaoIsVisible = false;
  
  public tipoBotao: string | null = null;
  
  public valueProduto: IDictionary = {};
  
  public appController = Loader.getInstance('AppController');

  public async openProduct({ component }: IEventParam<Datasource>) {
    const produto = component.currentRow.CDPRODUTO;
    
    if (produto) {
      const unidadeAdicional: GridEditable = Metadata.getInstance('gridConversao');
      
      const conditions = [
        {
          name: 'P_CDPRODUTO',
          operator: '=',
          value: produto,
        },
      ];
      unidadeAdicional.datasource.filter = conditions;
      await unidadeAdicional.datasource.get();
      this.cardUnidadeAdicional();
    } else {
      const card = Metadata.getInstance<Card>('cardConversao');
      card.isVisible = false;
      this.conversaoIsVisible = false;
    }
  }
  
  public setProducts() {
    const nomeProduto = Metadata.getInstance<Select>('NMPRODUTO');
    const cdFilial = this.appController.getCdCentral();

    nomeProduto.datasource.filter = [
      {
        name: 'CDFILIAL',
        operator: '=',
        value: cdFilial,
      },
    ];
    nomeProduto.datasource.get();
  }
  
  public addUnidade() {
    this.tipoBotao = 'buttonUnidade';
    if (this.utilities.centralIsValid()) {
      if (!this.modalConversao) {
        this.createModalAddConversao().then(() => {
          this.modalConversao.show();
        });
      } else {
        this.modalConversao.show();
      }
    }
  }
  
  public addConversao() {
    this.tipoBotao = 'buttonConversao';
    if (this.utilities.centralIsValid()) {
      if (!this.modalConversao) {
        this.createModalAddConversao().then(() => {
          this.modalConversao.show();
        });
      } else {
        this.modalConversao.show();
      }
    }
  }
  
  public closeConversaoModalAdd() {
    this.modalConversao.hide();
  }
  
  public closeUnidadeModalAdd() {
    this.modalUnidade.hide();
  }
  
  private async createModalAddConversao() {
    this.modalConversao = await ModalService.createFromJson('planejamento/produto/modals/ger01000_addConversao', true);
  }
  
  public reloadUnidade() {
    const gridConversao = Metadata.getInstance<GridEditable>('gridUnidadeAdicionalProduto');
    this.gridEditController.reload({ component: gridConversao });
    this.closeCardConversao();
  }
  
  private cardUnidadeAdicional() {
    const card = Metadata.getInstance<Card>('cardConversao');
    try {
      if (!this.gridEditController.newRow) {
        card.isVisible = true;
        this.conversaoIsVisible = true;
      }
    } catch (Error) {
      card.isVisible = false;
      this.conversaoIsVisible = false;
    }
  }
  
  public salvaConversao({ component }: IEventParam<Button>) {
    const gridConversao = Metadata.getInstance<GridEditable>('gridConversao');
    const formConversao = Metadata.getInstance<Form>('formConversaoUnidade');
    const row = formConversao.value;

    if (formConversao.validate()) {
      row.IDTIPOPE = '*';
      row.SGUNIDADE = row.SGUNIDADECONV;
      
      if (this.tipoBotao === 'buttonConversao') {
        row.CDPRODUTO = this.valueProduto.CDPRODUTO;
        row.NMPRODUTO = this.valueProduto.NMPRODUTO;
      } else {
        row.CDPRODUTO = row.NMPRODUTO.CDPRODUTO;
        row.NMPRODUTO = row.NMPRODUTO.NMPRODUTO;
      }

      this.utilities.activeLoadingButton(component);
      Http.post('/unidadeAdicionalProduto/save', { requestType: 'Row', row }).then((response) => {
        if (response.data.dataset.save[0]) {
          gridConversao.datasource.get();
          this.utilities.showAlert('alert', 'pme::salvoComSucesso', 'success');
          if (this.tipoBotao === 'buttonUnidade') this.reloadUnidade();
          this.closeConversaoModalAdd();
        }
      }).catch((error: string) => {
        this.utilities.showAlert('alert', `pme::${error}`, 'error');
      }).finally(() => {
        this.utilities.disableLoadingButton(component);
      });
    }
  }
  
  private setMainUnit() {
    const formProduto = Metadata.getInstance<Form>('formConversaoUnidade');
    const unidPrincipal = Metadata.getInstance<TextInput>('SGUNIDADE');
    const unidSecundaria = Metadata.getInstance<TextInput>('SGUNIDPARA');

    if (this.tipoBotao === 'buttonUnidade' && unidPrincipal) {
      unidPrincipal.value = formProduto.value.NMPRODUTO.DSUNIDADE;
      unidSecundaria.value = [];
    }
  }
  
  private setParams(rowConversao: IDictionary, valorX: IDictionary) {
    if (rowConversao.SGUNIDPARA) {
      rowConversao.DSUNIDADE = rowConversao.SGUNIDPARA.DSUNIDADE;
      rowConversao.SGUNIDADECONV = rowConversao.SGUNIDPARA.SGUNIDADE;
      valorX.disabled = false;
    } else {
      valorX.disabled = true;
      valorX.value = null;
      this.setValueUnPrinc();
    }
  }
  
  public setOpcRelacao() {
    let dsUnidade = '';
    const gridUnidadeAdicional = Metadata.getInstance<GridEditable>('gridUnidadeAdicionalProduto');
    const formConversao = Metadata.getInstance<Form>('formConversaoUnidade');
    const rowConversao = formConversao.value;
    const opcRelacao = formConversao.children.find((name) => name.name === 'OPCRELACAO');
    const valorX = Metadata.getInstance<TextInput>('VRFTCONVER');
    if (this.tipoBotao === 'buttonConversao') {
      dsUnidade = gridUnidadeAdicional.datasource.currentRow.DSUNIDADE;
    } else if (this.tipoBotao === 'buttonUnidade') {
      dsUnidade = rowConversao.NMPRODUTO.DSUNIDADE;
    }

    this.setParams(rowConversao, valorX);

    if (formConversao.value.SGUNIDPARA && opcRelacao && valorX.value) {
      opcRelacao.datasource.data[0].name = `1 ${dsUnidade} ${I18n.translate('pme::equivaleA')} 
        ${valorX.value} ${rowConversao.DSUNIDADE}(s)`;
      opcRelacao.datasource.data[1].name = `1 ${rowConversao.DSUNIDADE} 
        ${I18n.translate('pme::equivaleA')} ${valorX.value} ${dsUnidade}(s)`;
    } else if (formConversao.value.SGUNIDPARA && opcRelacao) {
      opcRelacao.datasource.data[0].name = `1 ${dsUnidade} ${I18n.translate('pme::equivaleA')} X 
        ${rowConversao.DSUNIDADE}(s)`;
      opcRelacao.datasource.data[1].name = `1 ${rowConversao.DSUNIDADE} 
        ${I18n.translate('pme::equivaleA')} X ${dsUnidade}(s)`;
    }
  }
  
  public async setValueUnPrinc() {
    const unidPrinc = Metadata.getInstance<TextInput>('SGUNIDADE');
    const nmProduto = Metadata.getInstance<Select>('NMPRODUTO');
    const inputProduto = Metadata.getInstance<TextInput>('PRODUTO_INPUT');
    const formConversao = Metadata.getInstance<Form>('formConversaoUnidade');
    const opcRelacao = formConversao.children.find((name) => name.name === 'OPCRELACAO');
    const gridUnidadeAdicional = Metadata.getInstance<GridEditable>('gridUnidadeAdicionalProduto');
    const sgUnidade = gridUnidadeAdicional.datasource.currentRow.SGUNIDADE;
    const dsUnidade = gridUnidadeAdicional.datasource.currentRow.DSUNIDADE
      ? gridUnidadeAdicional.datasource.currentRow.DSUNIDADE : '';
    
    if (this.tipoBotao === 'buttonConversao') {
      const cdFilial = this.appController.getCdCentral();
  
      nmProduto.datasource.filter = [
        {
          name: 'CDFILIAL',
          operator: '=',
          value: cdFilial,
        },
      ];
      await nmProduto.datasource.get();
      const campoProduto = {
        CDARVPROD: gridUnidadeAdicional.datasource.currentRow.CDARVPROD,
        NMPRODUTO: gridUnidadeAdicional.datasource.currentRow.NMPRODUTO,
        CDPRODUTO: gridUnidadeAdicional.datasource.currentRow.CDPRODUTO,
      };
      
      nmProduto.isVisible = false;
      nmProduto.removeValidation('required');
      inputProduto.isVisible = true;
      inputProduto.value = `${this.utilities.getValueWithMaskProduto(campoProduto.CDARVPROD)} | ${campoProduto.NMPRODUTO}`;
      this.valueProduto = campoProduto;
      unidPrinc.value = `${sgUnidade} | ${dsUnidade}`;
      
      if (opcRelacao) {
        opcRelacao.datasource.data[0].name = `1 ${dsUnidade} ${I18n.translate('pme::equivaleA')} X(s)`;
        opcRelacao.datasource.data[1].name = `X ${I18n.translate('pme::equivaleA')} 1 ${dsUnidade}(s)`;
      }
    } else {
      nmProduto.disabled = false;
      nmProduto.addValidation('required', { message: 'pme::campoObrigatorio' });
      inputProduto.isVisible = false;
      
      if (opcRelacao) {
        opcRelacao.datasource.data[0].name = `1 ${dsUnidade} ${I18n.translate('pme::equivaleA')} X(s)`;
        opcRelacao.datasource.data[1].name = `X ${I18n.translate('pme::equivaleA')} 1 ${dsUnidade}(s)`;
      }
    }
  }
  
  setRelacao({ row }: IDictionary) {
    return `1 ${row.DSUNIDADE} ${I18n.translate('pme::equivaleA')} ${row.VRFATOCONVES} ${row.DSUNIDPROD} (s)`;
  }
  
  public deleteCheckedRowsUnidade() {
    const gridUnidadeAdicional: GridEditable = Metadata.getInstance('gridUnidadeAdicionalProduto');
    const checkedRows = gridUnidadeAdicional.selectedRows;
    
    if (checkedRows.length === 0) this.utilities.showAlert('alert', 'pme::nenhumRegSelecionado', 'warning');
    else if (checkedRows.length > 1) {
      DialogService.show({
        name: 'confirmDelete',
        type: 'warning',
        title: 'pme::atencao',
        text: 'pme::excluirProdutosConversao',
        buttons: [
          {
            name: 'yesUnAdicional',
            component: 'ZdButton',
            label: 'pme::sim',
            events: { click: () => this.deleteUnidade() },
            outline: true,
          },
          {
            name: 'noUnAdicional',
            component: 'ZdButton',
            label: 'pme::nao',
            events: { click: () => DialogService.hide() },
          },
          {
            name: 'progressLine',
            component: 'ZdProgress',
            label: 'Progress',
            indeterminate: true,
            cssClass: 'loading-progress-line',
          },
        ],
      });
    } else if (checkedRows.length === 1) {
      const produto = Mask.getValueWithMask(checkedRows[0].CDPRODUTO, this.utilities.maskProd());
      const produtoConcat = `${produto} - ${checkedRows[0].NMPRODUTO}`;
      DialogService.show({
        name: 'confirmDelete',
        type: 'warning',
        title: 'pme::atencao',
        text: I18n.translate('pme::excluirProdutoConversao', { PARAM1: produtoConcat }),
        buttons: [
          {
            name: 'yesUnAdicional',
            component: 'ZdButton',
            label: 'pme::sim',
            events: { click: () => this.deleteUnidade() },
            outline: true,
          },
          {
            name: 'noUnAdicional',
            component: 'ZdButton',
            label: 'pme::nao',
            events: { click: () => DialogService.hide() },
          },
          {
            name: 'progressLine',
            component: 'ZdProgress',
            label: 'Progress',
            indeterminate: true,
            cssClass: 'loading-progress-line',
          },
        ],
      });
    }
  }
  
  public deleteUnidade() {
    const gridUnidadeAdicional: GridEditable = Metadata.getInstance('gridUnidadeAdicionalProduto');
    const button = Metadata.getInstance<Button>('yesUnAdicional');
    const checkedRows = gridUnidadeAdicional.selectedRows;
    if (checkedRows.length) {
      this.utilities.activeLoadingButton(button);
      Http.post('/unidadeAdicionalProduto/deleteUnidade', { requestType: 'Row', checkedRows }).then(() => {
        gridUnidadeAdicional.datasource.get();
        gridUnidadeAdicional.selectedRows = [];
        this.utilities.showAlert('alertSuccess', 'pme::excluidoComSucesso', 'success');
      }).finally(() => {
        DialogService.hide();
        this.utilities.disableLoadingButton(button);
      });
    }
  }
  
  public deleteCheckedRowsConversao() {
    const gridConversao: GridEditable = Metadata.getInstance('gridConversao');
    const checkedRows = gridConversao.selectedRows;
    
    if (checkedRows.length === 0) this.utilities.showAlert('alert', 'pme::nenhumRegSelecionado', 'warning');
    else {
      DialogService.show({
        name: 'confirmDelete',
        type: 'warning',
        title: 'pme::atencao',
        text: 'pme::desejaExcluirRegistros',
        buttons: [
          {
            name: 'yesConversao',
            component: 'ZdButton',
            label: 'pme::sim',
            events: { click: () => this.deleteConversao() },
            outline: true,
          },
          {
            name: 'noConversao',
            component: 'ZdButton',
            label: 'pme::nao',
            events: { click: () => DialogService.hide() },
          },
          {
            name: 'progressLine',
            component: 'ZdProgress',
            label: 'Progress',
            indeterminate: true,
            cssClass: 'loading-progress-line',
          },
        ],
      });
    }
  }
  
  public deleteConversao() {
    const gridConversao: GridEditable = Metadata.getInstance('gridConversao');
    const card = Metadata.getInstance<Card>('cardConversao');
    const checkedRows = gridConversao.selectedRows;
    const button = Metadata.getInstance<Button>('yesConversao');

    if (checkedRows.length) {
      this.utilities.activeLoadingButton(button);
      Http.post('/unidadeAdicionalProduto/deleteConversao', { requestType: 'Row', checkedRows }).then(() => {
        gridConversao.cancelEditedRows();
        gridConversao.datasource.get();
        gridConversao.selectedRows = [];
        DialogService.hide();
        this.utilities.showAlert('alertError', 'pme::excluidoComSucesso', 'success');
      }).catch((error: string) => {
        this.utilities.showAlert('alert', `pme::${error}`, 'error');
        DialogService.hide();
      }).finally(() => {
        card.isVisible = false;
        this.conversaoIsVisible = false;
        this.reloadUnidade();
        this.utilities.disableLoadingButton(button);
      });
    }
  }
  
  public closeCardConversao() {
    const card = Metadata.getInstance<Card>('cardConversao');
    
    card.isVisible = false;
    this.conversaoIsVisible = false;
  }
  
  get gridColsReturnValue(): number {
    return this.conversaoIsVisible ? 6 : 12;
  }
  
  public unidadeComponentProps = {
    component: 'ZdSelect',
    dataText: [
      {
        name: 'CDPRODUTO',
      },
      'NMPRODUTO',
    ],
    formatterDataText: [
      {
        name: 'CDARVPROD',
        mask: (Config.masks.productMask as PmeMask).pattern,
      },
      'NMPRODUTO',
    ],
    dataValueOut: [
      {
        column: 'CDPRODUTO',
        columnOnGrid: 'NMPRODUTO',
      },
    ],
    dataValue: 'CDPRODUTO',
    dense: true,
    datasource: {
      uniqueKey: 'CDPRODUTO',
      route: '/produto',
      type: 'rest',
      limit: 10,
      lazyLoad: true,
    },
    validations: {
      required: {
        message: 'pme::campoObrigatorio',
      },
    },
  };
}
